@@ -52,9 +52,10 @@
 
 /* these addrs must be in low memory so that they are mapped by the */
 /* kernel at startup */
+#define TBOOT_LOWMEM_START           0x60000
 
 /* address/size for memory-resident serial log (when enabled) */
-#define TBOOT_SERIAL_LOG_ADDR        0x60000
+#define TBOOT_SERIAL_LOG_ADDR        TBOOT_LOWMEM_START
 #define TBOOT_SERIAL_LOG_SIZE        0x08000
 
 /* address/size for modified e820 table */
@@ -62,14 +63,21 @@
 				      TBOOT_SERIAL_LOG_SIZE)
 #define TBOOT_E820_COPY_SIZE         0x01800
 
+/* address/size for modified EFI memory map table */
+#define TBOOT_EFI_MEMMAP_COPY_ADDR   (TBOOT_E820_COPY_ADDR + \
+                                      TBOOT_E820_COPY_SIZE)
+#define TBOOT_EFI_MEMMAP_COPY_SIZE   0x08000
+
 /* address/size for modified VMM/kernel command line */
-#define TBOOT_KERNEL_CMDLINE_ADDR    (TBOOT_E820_COPY_ADDR + \
-				      TBOOT_E820_COPY_SIZE)
+#define TBOOT_KERNEL_CMDLINE_ADDR    (TBOOT_EFI_MEMMAP_COPY_ADDR + \
+                                      TBOOT_EFI_MEMMAP_COPY_SIZE)
 #define TBOOT_KERNEL_CMDLINE_SIZE    0x0400
 
+#define TBOOT_LOWMEM_END             (TBOOT_KERNEL_CMDLINE_ADDR + \
+                                      TBOOT_KERNEL_CMDLINE_SIZE)
 
 #ifndef NR_CPUS
-#define NR_CPUS     256
+#define NR_CPUS     1024
 #endif
 
 #ifdef __ASSEMBLY__
@@ -98,11 +106,26 @@ extern char _end[];              /* end of tboot */
 #define __packed   __attribute__ ((packed))
 
 /* tboot log level */
+#ifdef NO_TBOOT_LOGLVL
+#define TBOOT_NONE
+#define TBOOT_ERR
+#define TBOOT_WARN
+#define TBOOT_INFO
+#define TBOOT_DETA
+#define TBOOT_ALL
+#else /* NO_TBOOT_LOGLVL */
 #define TBOOT_NONE       "<0>"
 #define TBOOT_ERR        "<1>"
 #define TBOOT_WARN       "<2>"
 #define TBOOT_INFO       "<3>"
-#define TBOOT_ALL        "<4>"
+#define TBOOT_DETA       "<4>"
+#define TBOOT_ALL        "<5>"
+#endif /* NO_TBOOT_LOGLVL */
+
+/* Framebuffer */
+#define FB_MAX_HRES 2560
+#define FB_MAX_VRES 1440
+#define FB_BPP 32
 
 #endif /* __CONFIG_H__ */
 
