@@ -53,8 +53,6 @@
 #endif
 
 static struct acpi_rsdp *rsdp;
-static struct acpi_table_header *g_dmar_table;
-static __data bool g_hide_dmar;
 
 static void dump_gas(const char *reg_name,
                      const tboot_acpi_generic_address_t *reg)
@@ -108,7 +106,7 @@ static bool find_rsdp_in_range(void *start, void *end)
     for ( ; start < end; start += RSDP_BOUNDARY ) {
         rsdp = (struct acpi_rsdp *)start;
 
-        if ( memcmp(rsdp->rsdp1.signature, RSDP_SIG,
+        if ( tb_memcmp(rsdp->rsdp1.signature, RSDP_SIG,
                     sizeof(rsdp->rsdp1.signature)) == 0 ) {
             if ( verify_acpi_checksum((uint8_t *)rsdp, RSDP_CHKSUM_LEN) ) {
                 printk(TBOOT_DETA"RSDP (v%u, %.6s) @ %p\n", rsdp->rsdp1.revision,
@@ -189,7 +187,7 @@ static struct acpi_table_header *find_table(const char *table_name)
               curr_table < (uint64_t *)((void *)xsdt + xsdt->hdr.length);
               curr_table++ ) {
             table = (struct acpi_table_header *)(uintptr_t)*curr_table;
-            if ( memcmp(table->signature, table_name,
+            if ( tb_memcmp(table->signature, table_name,
                         sizeof(table->signature)) == 0 )
                 return table;
         }
@@ -206,86 +204,31 @@ static struct acpi_table_header *find_table(const char *table_name)
               curr_table < (uint32_t *)((void *)rsdt + rsdt->hdr.length);
               curr_table++ ) {
             table = (struct acpi_table_header *)(uintptr_t)*curr_table;
-            if ( memcmp(table->signature, table_name,
+            if ( tb_memcmp(table->signature, table_name,
                         sizeof(table->signature)) == 0 )
                 return table;
         }
     }
 
-    printk(TBOOT_ERR"cann't find %s table.\n", table_name);
+    printk(TBOOT_ERR"can't find %s table.\n", table_name);
     return NULL;
 }
 
-static struct acpi_dmar *get_vtd_dmar_table(void)
-{
-    return (struct acpi_dmar *)find_table(DMAR_SIG);
-}
-
-bool save_vtd_dmar_table(void)
-{
-    /* find DMAR table and save it */
-    g_dmar_table = (struct acpi_table_header *)get_vtd_dmar_table();
-
-    printk(TBOOT_DETA"DMAR table @ %p saved.\n", g_dmar_table);
-    return true;
-}
-
-bool restore_vtd_dmar_table(void)
-{
-    struct acpi_table_header *hdr;
-
-    g_hide_dmar = false;
-
-    /* find DMAR table first */
-    hdr = (struct acpi_table_header *)get_vtd_dmar_table();
-    if ( hdr != NULL ) {
-        printk(TBOOT_DETA"DMAR table @ %p is still there, skip restore step.\n", hdr);
-        return true;
-    }
-
-    /* check saved DMAR table */
-    if ( g_dmar_table == NULL ) {
-        printk(TBOOT_ERR"No DMAR table saved, abort restore step.\n");
-        return false;
-    }
-
-    /* restore DMAR if needed */
-    memcpy(g_dmar_table->signature, DMAR_SIG, sizeof(g_dmar_table->signature));
-
-    /* need to hide DMAR table while resume from S3 */
-    g_hide_dmar = true;
-    printk(TBOOT_DETA"DMAR table @ %p restored.\n", hdr);
-    return true;
-}
-
-bool remove_vtd_dmar_table(void)
-{
-    struct acpi_table_header *hdr;
-
-    /* check whether it is needed */
-    if ( !g_hide_dmar ) {
-        printk(TBOOT_DETA"No need to hide DMAR table.\n");
-        return true;
-    }
-
-    /* find DMAR table */
-    hdr = (struct acpi_table_header *)get_vtd_dmar_table();
-    if ( hdr == NULL ) {
-        printk(TBOOT_DETA"No DMAR table, skip remove step.\n");
-        return true;
-    }
-
-    /* remove DMAR table */
-    hdr->signature[0] = '\0';
-    printk(TBOOT_DETA"DMAR table @ %p removed.\n", hdr);
-    return true;
-}
-
 static struct acpi_madt *get_apic_table(void)
 {
     return (struct acpi_madt *)find_table(MADT_SIG);
 }
 
+uint32_t get_madt_apic_base(void)
+{
+    struct acpi_madt *madt = get_apic_table();
+    if ( madt == NULL ) {
+        printk(TBOOT_ERR"no MADT table found\n");
+        return 0;
+    }
+    return (uint32_t)madt->local_apic_address;
+}
+
 struct acpi_table_ioapic *get_acpi_ioapic_table(void)
 {
     struct acpi_madt *madt = get_apic_table();
@@ -318,6 +261,11 @@ struct acpi_mcfg *get_acpi_mcfg_table(void)
     return (struct acpi_mcfg *)find_table(MCFG_SIG);
 }
 
+struct acpi_dmar *get_vtd_dmar_table(void)
+{
+    return (struct acpi_dmar *)find_table(DMAR_SIG);
+}
+
 static bool write_to_reg(const tboot_acpi_generic_address_t *reg,
                          uint32_t val)
 {
