@@ -56,6 +56,8 @@
 #include <hash.h>
 #include <cmdline.h>
 #include <acpi.h>
+#include <vtd.h>
+#include <efi_memmap.h>
 #include <txt/txt.h>
 #include <txt/config_regs.h>
 #include <txt/mtrrs.h>
@@ -89,6 +91,8 @@ extern void print_event(const tpm12_pcr_event_t *evt);
 extern void print_event_2(void *evt, uint16_t alg);
 extern uint32_t print_event_2_1(void *evt);
 
+extern void __enable_nmi(void);
+
 /*
  * this is the structure whose addr we'll put in TXT heap
  * it needs to be within the MLE pages, so force it to the .text section
@@ -178,7 +182,7 @@ static void *build_mle_pagetable(uint32_t mle_start, uint32_t mle_size)
     /* place ptab_base below MLE */
     ptab_size = sizeof(g_mle_pt);
     ptab_base = &g_mle_pt;
-    memset(ptab_base, 0, ptab_size);
+    tb_memset(ptab_base, 0, ptab_size);
     printk(TBOOT_DETA"ptab_size=%x, ptab_base=%p\n", ptab_size, ptab_base);
 
     pg_dir_ptr_tab = ptab_base;
@@ -214,7 +218,7 @@ static void *init_event_log(void)
     os_mle_data_t *os_mle_data = get_os_mle_data_start(get_txt_heap());
     g_elog = (event_log_container_t *)&os_mle_data->event_log_buffer;
 
-    memcpy((void *)g_elog->signature, EVTLOG_SIGNATURE,
+    tb_memcpy((void *)g_elog->signature, EVTLOG_SIGNATURE,
            sizeof(g_elog->signature));
     g_elog->container_ver_major = EVTLOG_CNTNR_MAJOR_VER;
     g_elog->container_ver_minor = EVTLOG_CNTNR_MINOR_VER;
@@ -249,16 +253,6 @@ static void init_evtlog_desc(heap_event_log_ptr_elt2_t *evt_log)
     os_mle_data_t *os_mle_data = get_os_mle_data_start(get_txt_heap());
     struct tpm_if *tpm = get_tpm();
     switch (tpm->extpol) {
-    case TB_EXTPOL_AGILE:
-        for (i=0; i<evt_log->count; i++) {
-            evt_log->event_log_descr[i].alg = tpm->algs_banks[i];
-            evt_log->event_log_descr[i].phys_addr =
-                    (uint64_t)(unsigned long)(os_mle_data->event_log_buffer + i*4096);
-            evt_log->event_log_descr[i].size = 4096;
-            evt_log->event_log_descr[i].pcr_events_offset = 0;
-            evt_log->event_log_descr[i].next_event_offset = 0;
-        }
-        break;
     case TB_EXTPOL_EMBEDDED:
         for (i=0; i<evt_log->count; i++) {
             evt_log->event_log_descr[i].alg = tpm->algs[i];
@@ -330,13 +324,10 @@ static void init_os_sinit_ext_data(heap_ext_data_element_t* elts)
         printk(TBOOT_DETA"heap_ext_data_element SIZE = %d \n", elt->size);
     }  else if ( log_type == EVTLOG_TPM2_LEGACY ) {
         g_elog_2 = (heap_event_log_ptr_elt2_t *)elt->data;
-        if ( tpm->extpol == TB_EXTPOL_AGILE )
-            g_elog_2->count = tpm->banks;
+        if ( tpm->extpol == TB_EXTPOL_EMBEDDED )
+            g_elog_2->count = tpm->alg_count;
         else
-            if ( tpm->extpol == TB_EXTPOL_EMBEDDED )
-                g_elog_2->count = tpm->alg_count;
-            else
-                g_elog_2->count = 1;
+            g_elog_2->count = 1;
         init_evtlog_desc(g_elog_2);
         elt->type = HEAP_EXTDATA_TYPE_TPM_EVENT_LOG_PTR_2;
         elt->size = sizeof(*elt) + sizeof(u32) +
@@ -362,7 +353,7 @@ bool evtlog_append_tpm12(uint8_t pcr, tb_hash_t *hash, uint32_t type)
 
     next->pcr_index = pcr;
     next->type = type;
-    memcpy(next->digest, hash, sizeof(next->digest));
+    tb_memcpy(next->digest, hash, sizeof(next->digest));
     next->data_size = 0;
 
     g_elog->next_event_offset += sizeof(*next) + next->data_size;
@@ -436,7 +427,7 @@ bool evtlog_append_tpm2_legacy(uint8_t pcr, uint16_t alg, tb_hash_t *hash, uint3
     next += sizeof(u32);
     *((u32 *)next) = type;
     next += sizeof(u32);
-    memcpy((uint8_t *)next, hash, hash_size);
+    tb_memcpy((uint8_t *)next, hash, hash_size);
     next += hash_size;
     *((u32 *)next) = 0;
     cur_desc->next_event_offset += 3*sizeof(uint32_t) + hash_size; 
@@ -476,8 +467,8 @@ bool evtlog_append_tpm2_tcg(uint8_t pcr, uint32_t type, hash_list_t *hl)
         return false;
     }
 
-    event = (tcg_pcr_event2*)(void *)(unsigned long)g_elog_2_1->phys_addr +
-        g_elog_2_1->next_record_offset;
+    event = (tcg_pcr_event2*)(void *)(unsigned long)(g_elog_2_1->phys_addr +
+        g_elog_2_1->next_record_offset);
     event->pcr_index = pcr;
     event->event_type = type;
     event->event_size = 0;  // No event data passed by tboot.
@@ -489,7 +480,7 @@ bool evtlog_append_tpm2_tcg(uint8_t pcr, uint32_t type, hash_list_t *hl)
         *((uint16_t *)hash_entry) = hl->entries[i].alg; // TPMT_HA_1.hash_alg
         hash_entry += sizeof(uint16_t);
         hash_size = get_hash_size(hl->entries[i].alg);  // already checked above
-        memcpy(hash_entry, &(hl->entries[i].hash), hash_size);
+        tb_memcpy(hash_entry, &(hl->entries[i].hash), hash_size);
         hash_entry += hash_size;
     }
 
@@ -527,6 +518,42 @@ bool evtlog_append(uint8_t pcr, hash_list_t *hl, uint32_t type)
 __data uint32_t g_using_da = 0;
 __data acm_hdr_t *g_sinit = 0;
 
+static void configure_vtd(void)
+{
+    uint32_t remap_length;
+    struct dmar_remapping *dmar_remap = vtd_get_dmar_remap(&remap_length);
+
+    if (dmar_remap == NULL) {
+        printk("cannot get DMAR remapping structures, skipping configuration\n");
+        return;
+    } else {
+        printk("configuring DMAR remapping\n");
+    }
+
+    struct dmar_remapping *iter, *next;
+    struct dmar_remapping *end = ((void *)dmar_remap) + remap_length;
+
+    for (iter = dmar_remap; iter < end; iter = next) {
+        next = (void *)iter + iter->length;
+        if (iter->length == 0) {
+            /* Avoid looping forever on bad ACPI tables */
+            printk("    invalid 0-length structure\n");
+            break;
+        } else if (next > end) {
+            /* Avoid passing table end */
+            printk("    record passes table end\n");
+            break;
+        }
+
+        if (iter->type == DMAR_REMAPPING_DRHD) {
+            if (!vtd_disable_dma_remap(iter)) {
+                printk("    vtd_disable_dma_remap failed!\n");
+            }
+        }
+    }
+
+}
+
 /*
  * sets up TXT heap
  */
@@ -550,7 +577,7 @@ static txt_heap_t *init_txt_heap(void *ptab_base, acm_hdr_t *sinit, loader_ctx *
     os_mle_data_t *os_mle_data = get_os_mle_data_start(txt_heap);
     size = (uint64_t *)((uint32_t)os_mle_data - sizeof(uint64_t));
     *size = sizeof(*os_mle_data) + sizeof(uint64_t);
-    memset(os_mle_data, 0, sizeof(*os_mle_data));
+    tb_memset(os_mle_data, 0, sizeof(*os_mle_data));
     os_mle_data->version = 3;
     os_mle_data->lctx_addr = lctx->addr;
     os_mle_data->saved_misc_enable_msr = rdmsr(MSR_IA32_MISC_ENABLE);
@@ -571,7 +598,7 @@ static txt_heap_t *init_txt_heap(void *ptab_base, acm_hdr_t *sinit, loader_ctx *
     os_sinit_data_t *os_sinit_data = get_os_sinit_data_start(txt_heap);
     size = (uint64_t *)((uint32_t)os_sinit_data - sizeof(uint64_t));
     *size = calc_os_sinit_data_size(version);
-    memset(os_sinit_data, 0, *size);
+    tb_memset(os_sinit_data, 0, *size);
     os_sinit_data->version = version;
 
     /* this is phys addr */
@@ -599,7 +626,7 @@ static txt_heap_t *init_txt_heap(void *ptab_base, acm_hdr_t *sinit, loader_ctx *
                    lcp_size);
             return NULL;
         }
-        memcpy(os_mle_data->lcp_po_data, lcp_base, lcp_size);
+        tb_memcpy(os_mle_data->lcp_po_data, lcp_base, lcp_size);
         os_sinit_data->lcp_po_base = (unsigned long)&os_mle_data->lcp_po_data;
         os_sinit_data->lcp_po_size = lcp_size;
     }
@@ -640,7 +667,7 @@ static txt_heap_t *init_txt_heap(void *ptab_base, acm_hdr_t *sinit, loader_ctx *
                 os_sinit_data->efi_rsdt_ptr = (uint64_t) rsdp->rsdp1.rsdt;
             } else {
                 /* rsdp */
-                memcpy((void *)&g_rsdp, rsdp, sizeof(struct acpi_rsdp));
+                tb_memcpy((void *)&g_rsdp, rsdp, sizeof(struct acpi_rsdp));
                 os_sinit_data->efi_rsdt_ptr = (uint64_t)((uint32_t)&g_rsdp);
             }
         } else {
@@ -652,7 +679,7 @@ static txt_heap_t *init_txt_heap(void *ptab_base, acm_hdr_t *sinit, loader_ctx *
         
     /* capabilities : choose DA/LG */
     os_sinit_data->capabilities.pcr_map_no_legacy = 1;
-    if ( sinit_caps.pcr_map_da && get_tboot_prefer_da() )
+    if ( sinit_caps.pcr_map_da && (get_tboot_prefer_da() || sinit_caps.cbnt_supported) )
         os_sinit_data->capabilities.pcr_map_da = 1;
     else if ( !sinit_caps.pcr_map_no_legacy )
         os_sinit_data->capabilities.pcr_map_no_legacy = 0;
@@ -671,11 +698,11 @@ static txt_heap_t *init_txt_heap(void *ptab_base, acm_hdr_t *sinit, loader_ctx *
     /* PCR mapping selection MUST be zero in TPM2.0 mode
      * since D/A mapping is the only supported by TPM2.0 */
     if ( tpm->major >= TPM20_VER_MAJOR ) {
-        os_sinit_data->flags = (tpm->extpol == TB_EXTPOL_AGILE) ? 0 : 1;
+        os_sinit_data->flags = 1;
         os_sinit_data->capabilities.pcr_map_no_legacy = 0;
         os_sinit_data->capabilities.pcr_map_da = 0;
         g_using_da = 1;
-    }   
+    }
 
     /* Event log initialization */
     if ( os_sinit_data->version >= 6 )
@@ -701,8 +728,9 @@ static void txt_wakeup_cpus(void)
 
     /* enable SMIs on BSP before waking APs (which will enable them on APs)
        because some SMM may take immediate SMI and hang if AP gets in first */
-    printk(TBOOT_DETA"enabling SMIs on BSP\n");
+    printk(TBOOT_DETA"enabling SMIs and NMI on BSP\n");
     __getsec_smctrl();
+    __enable_nmi();
 
     atomic_set(&ap_wfs_count, 0);
 
@@ -811,6 +839,8 @@ tb_error_t txt_launch_environment(loader_ctx *lctx)
     if ( mle_ptab_base == NULL )
         return TB_ERR_FATAL;
 
+    configure_vtd();
+
     /* initialize TXT heap */
     txt_heap = init_txt_heap(mle_ptab_base, g_sinit, lctx);
     if ( txt_heap == NULL )
@@ -820,19 +850,25 @@ tb_error_t txt_launch_environment(loader_ctx *lctx)
     os_mle_data = get_os_mle_data_start(txt_heap);
     save_mtrrs(&(os_mle_data->saved_mtrr_state));
 
+    /* 
+     * Disable VGA logging when using framebuffer. Writing to it will be
+     * extreme slow when memory is set to UC.
+     */
+    if (get_framebuffer_info(g_ldr_ctx) != NULL) {
+        printk(TBOOT_INFO"Disabling VGA logging before GETSEC[SENTER]\n");
+        printk_disable_vga();
+    }
+
+    /*
+    * If memlog decide to compress logs after setting MTRRs,
+    * it will take very much time. Better do it now.
+    */
+    printk_flush();
+
     /* set MTRRs properly for AC module (SINIT) */
     if ( !set_mtrrs_for_acmod(g_sinit) )
         return TB_ERR_FATAL;
 
-   /* deactivate current locality */
-   if (g_tpm_family == TPM_IF_20_CRB ) {
-       printk(TBOOT_INFO"Relinquish CRB localility 0 before executing GETSEC[SENTER]...\n");
-	if (!tpm_relinquish_locality_crb(0)){
-		printk(TBOOT_INFO"Relinquish CRB locality 0 failed...\n");
-		apply_policy(TB_ERR_TPM_NOT_READY) ;
-	}
-   }
-
    /*{
    tpm_reg_loc_ctrl_t    reg_loc_ctrl;
    tpm_reg_loc_state_t  reg_loc_state;
@@ -876,6 +912,12 @@ bool txt_s3_launch_environment(void)
         init_evtlog_desc(g_elog_2);
     }
 
+    /*
+    * If memlog decide to compress logs after setting MTRRs,
+    * it will take very much time. Better do it now.
+    */
+    printk_flush();
+
     /* set MTRRs properly for AC module (SINIT) */
     set_mtrrs_for_acmod(g_sinit);
 
@@ -910,6 +952,12 @@ tb_error_t txt_launch_racm(loader_ctx *lctx)
     /*  - not needed by far since always reboot after RACM launch */
     //save_mtrrs(...);
 
+    /*
+    * If memlog decide to compress logs after setting MTRRs,
+    * it will take very much time. Better do it now.
+    */
+    printk_flush();
+
     /* set MTRRs properly for AC module (RACM) */
     if ( !set_mtrrs_for_acmod(racm) )
         return TB_ERR_FATAL;
@@ -1158,9 +1206,10 @@ void txt_cpu_wakeup(void)
     if ( !verify_stm(cpuid) )
         apply_policy(TB_ERR_POST_LAUNCH_VERIFICATION);
 
-    /* enable SMIs */
-    printk(TBOOT_DETA"enabling SMIs on cpu %u\n", cpuid);
+    /* enable SMIs and NMI */
+    printk(TBOOT_DETA"enabling SMIs and NMI on cpu %u\n", cpuid);
     __getsec_smctrl();
+    __enable_nmi();
 
     atomic_inc(&ap_wfs_count);
     if ( use_mwait() )
@@ -1186,6 +1235,9 @@ tb_error_t txt_protect_mem_regions(void)
            (base + size - 1));
     if ( !e820_protect_region(base, size, E820_RESERVED) )
         return TB_ERR_FATAL;
+    if (!efi_memmap_reserve(base, size)) {
+        return TB_ERR_FATAL;
+    }
 
     /* SINIT */
     base = read_pub_config_reg(TXTCR_SINIT_BASE);
@@ -1194,6 +1246,9 @@ tb_error_t txt_protect_mem_regions(void)
            (base + size - 1));
     if ( !e820_protect_region(base, size, E820_RESERVED) )
         return TB_ERR_FATAL;
+    if (!efi_memmap_reserve(base, size)) {
+        return TB_ERR_FATAL;
+    }
 
     /* TXT private space */
     base = TXT_PRIV_CONFIG_REGS_BASE;
@@ -1203,6 +1258,9 @@ tb_error_t txt_protect_mem_regions(void)
            base, (base + size - 1));
     if ( !e820_protect_region(base, size, E820_RESERVED) )
         return TB_ERR_FATAL;
+    if (!efi_memmap_reserve(base, size)) {
+        return TB_ERR_FATAL;
+    }
 
     /* ensure that memory not marked as good RAM by the MDRs is RESERVED in
        the e820 table */
@@ -1301,7 +1359,7 @@ bool get_parameters(getsec_parameters_t *params)
         return false;
     }
 
-    memset(params, 0, sizeof(*params));
+    tb_memset(params, 0, sizeof(*params));
     params->acm_max_size = DEF_ACM_MAX_SIZE;
     params->acm_mem_types = DEF_ACM_MEM_TYPES;
     params->senter_controls = DEF_SENTER_CTRLS;
